	;CPCBooster communication code.
	;V1.0 by Targhan/Arkos.


CPCB_InitCommand equ #f0
CPCB_ReceiveCommand equ #f1
CPCB_EndCommand equ #f2
CPCB_RewindCommand equ #f3
CPCB_AskFileSizeCommand equ #f4
CPCB_AskFileNameCommand equ #f5
CPCB_CreateFileCommand equ #f6
CPCB_AddDataToFileCommand equ #f7
CPCB_CloseFileCommand equ #f8

CPCB_InitDSKCommand equ #f9
CPCB_SendTrackCommand equ #fa
CPCB_NoMoreTrackCommand equ #fb


CPCB_InitConfirmedFromPC equ #80
CPCB_FileCreated equ #80



CPCB_TimeoutValue equ #4000





;Initialise CPCB, baudrate etc...,
;RET=Carry=1=ok 0=non detectee.
CPCB_Init
	ld bc,#ff00
	in a,(c)
	cp 170
	jr nz,CPCBError
	inc c
	in a,(c)
	cp 85
	jr nz,CPCBError

	ld c,#04		;Setting Baudrate to 115200.
	ld a,#05
	out (c),c
	out (c),a

	ld c,#07		;Asynchronous, no Parity, 1 bit stop, 8 bits carac.
	ld a,%00000110
	out (c),c
	out (c),a

	ld c,#0b		;Enables buffer.
	in a,(c)
	set 4,a
	out (c),a

	ld c,#1c		;Reset buffer.
	xor a
	out (c),a

	scf
	ret

CPCBError or a
	ret




;Essaye de communiquer l'init avec le PC. Il y a un timeout pour reessayer si erreur.
;RET=Carry=1=ok 0=communication failed.
CPCB_InitPC
	ld a,CPCB_InitCommand
	call CPCB_SendByte

	call CPCB_GetByte
	jr nc,CPCBError			;Not carry=timeout
	cp CPCB_InitConfirmedFromPC
	jr nz,CPCBError
	scf
	ret


;Recoit un octet de la CPCB (avec flash de couleur. Activer encre d'abord !)
;RET=Carry=1=OK et A=byte   Carry=0=timeout
CPCB_GetByte
	ld bc,#ff1c
	ld de,CPCB_TimeoutValue
CPCBGBLp dec de
	ld a,d
	or e
	jr z,CPCBError	;Timeout
	in a,(c)
	or a
	jr z,CPCBGBLp

	inc c
	in a,(c)

	scf
	ret



;Envoi un octet a la CPCB
;A=byte
CPCB_SendByte
	ld bc,#ff08
	out (c),a
	ret



;Envoi un paquet d'octet a la CPCB.
;Cette routine ne fait qu'envoyer ces octets. Il faut lancer une commande avant si on veut que le PC
;comprenne.
;HL=donnees
;DE=taille
CPCB_SendXBytes
	ld a,(hl)
	call CPCB_SendByte
	inc hl
	dec de
	ld a,e
	or d
	jr nz,CPCB_SendXBytes
	ret







;Recoit X octets de la CPCB vers MEMOIRE LINEAIRE.
;Cette routine prepare l'envoi et demande au PC de lui envoyer ces octets grace a la commande Receive.
;HL=Nombre d'octets a recevoir
;DE=Destination
CPCB_GetXBytes
	ld a,CPCB_ReceiveCommand
	call CPCB_SendByte
	ld a,l				;Send the size to receive.
	call CPCB_SendByte
	ld a,h
	call CPCB_SendByte
CPCB_GetXBLoop
	push de
CPCB_GetXBL2 call CPCB_GetByte
	jr nc,CPCB_GetXBL2		;Si Timeout, on recommence
	pop de
	ld (de),a
	inc de
	ld a,l
	or h
	dec hl
	jr nz,CPCB_GetXBLoop
	ret



;Quand transfert termine, envoi de commande de fin.
CPCB_SendEndCommand
	ld a,CPCB_EndCommand
	call CPCB_SendByte
	ret



;Demande de nom fichier. Recoit 12 bytes (nom+point+ext)
;HL=Destination
CPCB_AskFileName
	ld a,CPCB_AskFileNameCommand
	call CPCB_SendByte
	ld d,12
CPCB_AFNLoop
	push de
	call CPCB_GetByte
	pop de
	ld (hl),a
	inc hl
	dec d
	jr nz,CPCB_AFNLoop
	ret


;Demande de filesize. Recoit 4 bytes, du point faible au point fort
;HL=Destination
CPCB_AskFileSize
	ld a,CPCB_AskFileSizeCommand
	call CPCB_SendByte
	ld d,4
	jr CPCB_AFNLoop


;Le pointeur PC du fichier est remis au debut du fichier.
CPCB_RewindFile
	ld a,CPCB_RewindCommand
	call CPCB_SendByte
	ret



;Cree un fichier PC
;HL=filename (8+point+3)
;RET=Carry=1=Ok 0=Echec
CPCB_CreateFile
	ld a,CPCB_CreateFileCommand
	call CPCB_SendByte

	ld de,12
	call CPCB_SendXBytes

	call CPCB_GetByte
	jp nc,CPCBError			;Not carry=timeout
	cp CPCB_FileCreated
	jp nz,CPCBError
	scf
	ret



;Envoi une commande pour ajouter des datas a un fichier PC deja ouvert.
;HL=donnees
;DE=taille
CPCB_AddDataToFile
	ld a,CPCB_AddDataToFileCommand
	call CPCB_SendByte
	ld a,e
	call CPCB_SendByte
	ld a,d
	call CPCB_SendByte

	call CPCB_SendXBytes
	ret



;Fermeture du fichier PC
CPCB_CloseFile
	ld a,CPCB_CloseFileCommand
	call CPCB_SendByte
	ret


;Dis au PC de generer un DSK vide, pour l'instant.
CPCB_InitDSK
	ld a,CPCB_InitDSKCommand
	call CPCB_SendByte
	ret


;Previens le PC que les donnees d'une track (info+donnees) vont arriver
CPCB_SendTrack
	ld a,CPCB_SendTrackCommand
	call CPCB_SendByte
	ret

;Dis au PC qu'il n'y a plus de track a recevoir. Il peut donc ecrire le DSK sur le DD.
CPCB_NoMoreTrack
	ld a,CPCB_NoMoreTrackCommand
	call CPCB_SendByte
	ret





; [Grim

;Nique moins de registres (DE) et plus court
;Recoit un octet de la CPCB
;RET=Carry=1=OK et A=byte   Carry=0=timeout
;BC,AF modifis

CPCB_FastGetByte
							ld bc,#ff1c
							ld a,b
CPCBFGBLp 					dec a
							jp z,CPCBError	;Timeout
				
										; enventuellement mettre un "ld b,c" si &FF ralenti trop
							;djnz $-1 	; Ralentisseur  mettre si le timeout est trop court
							;dec b		; replace &ff dans B
				
							dw &70ED ; in (c) opcode non document, affecte seulement les flags celon la valeur du port
							jr z,CPCBFGBLp
				
							inc c
							in a,(c)
				;
							scf
							ret

; FAST VERSION
;Recoit X octets de la CPCB vers MEMOIRE LINEAIRE.
;HL=Nombre d'octets a recevoir
;DE=Destination
CPCB_FastGetXBytes
							ld a,CPCB_ReceiveCommand
							call CPCB_SendByte
							ld a,l				;Send the size to receive.
							call CPCB_SendByte
							ld a,h
							call CPCB_SendByte
CPCB_FastGetXBLoop
							call CPCB_FastGetByte
							ld (de),a
							inc de
							dec hl
							ld a,l
							or h
							jr nz,CPCB_FastGetXBLoop
							scf
							ret




	list
;**** fin CPCBooster
	;nolist